@@ -1,4 +1,4 @@
 #define MSBFIRST
 #define TRIPLE_BITS
 #define FIXEDBASE
-#include "../../../../hw/xfree86/xaa/xaaStipple.c"
+#include "./xaaStipple.c"
